/*
 * rtsp_handler.c
 * CVE-2007-0015, QuickTime RTSP URL Handler Stack-based Buffer Overflow
 *
 * Copyright (c) 2007 Landon Fuller <landonf@opendarwin.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Landon Fuller nor the names of any contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <CoreFoundation/CoreFoundation.h>
#include <ApplicationEnhancer/ApplicationEnhancer.h>

#include <sys/types.h>
#include <mach-o/dyld.h>

#include "rtsp_handler.h"

#define QTSC_PATH "/System/Library/QuickTime/QuickTimeStreaming.component/Contents/MacOS/QuickTimeStreaming"
#define RTSP_MAXLENGTH 270

static const char safeURL[] = "http://rtsp.example.com/example.mov";

/* Function pointer prototype */
typedef void *(*func_INet_ParseURLServer)(char *url, uint32_t length);

/* Original Function */
func_INet_ParseURLServer orig_INet_ParseURLServer = NULL;

/* Replacement Function */
void *guard_INet_ParseURLServer(char *url, uint32_t length) {
    if (length > RTSP_MAXLENGTH) {
        apeprintf("Someone probably just attempted to exploit CVE-2007-0015, url=%s\n", url);
        return orig_INet_ParseURLServer((char *) safeURL, sizeof(safeURL)-1);
    }

    return orig_INet_ParseURLServer(url, length);
}

void rtsp_handler_init(CFBundleRef bundle, CFStringRef inAPEToolsApplicationID) {
    const struct mach_header *qtsc_header;

    /* We load the QuickTime Streaming component here, to make sure we have the address to the function we're patching
     * we need. A waste of address space for processes that don't need it -- sorry! */
    qtsc_header = NSAddImage(QTSC_PATH, NSADDIMAGE_OPTION_NONE);
    if (!qtsc_header) {
        apeprintf("Failed to load QTSC image\n");
        return;
    }

    orig_INet_ParseURLServer = APEFindSymbol((struct mach_header *) qtsc_header, "_INet_ParseURLServer");
    if (!orig_INet_ParseURLServer) {
        apeprintf("Failed to locate INet_ParseURLServer symbol\n");
        return;
    }

    orig_INet_ParseURLServer = APEPatchCreate(orig_INet_ParseURLServer, &guard_INet_ParseURLServer);
}
